from gcs_cli.helpers import requires_login
from gcs_cli.parsing import globus_command
from gcs_cli.prettyprint import FORMAT_TEXT_RECORD, formatted_print
from gcs_cli.services.gcs import get_client

STANDARD_FIELDS = [
    ("Display Name", "display_name"),
    ("ID", "id"),
    ("Client ID", "client_id"),
    ("Subscription ID", "subscription_id"),
    ("Public", "public"),
    ("GCS Manager URL", "gcs_manager_url"),
    ("GCS Manager Custom URL", "gcs_manager_custom_url"),
    ("Network Use", "network_use"),
    ("Organization", "organization"),
    ("Department", "department"),
    ("Keywords", "keywords"),
    ("Description", "description"),
    ("Contact E-mail", "contact_email"),
    ("Contact Info", "contact_info"),
    ("Endpoint Info Link", "info_link"),
]

NETWORK_USE_CUSTOM_FIELDS = [
    ("Maximum Concurrency", "max_concurrency"),
    ("Preferred Concurrency", "preferred_concurrency"),
    ("Maximum Parallelism", "max_parallelism"),
    ("Preferred Parallelism", "preferred_parallelism"),
]


@globus_command("show", short_help="Show the endpoint definition")
@requires_login
def endpoint_show():
    """
    Show the endpoint to which you are currently logged in
    """
    client = get_client()

    res = client.get_endpoint()

    fields = STANDARD_FIELDS
    if res.get("network_use") == "custom":
        fields += NETWORK_USE_CUSTOM_FIELDS

    # walk the list of all known fields and reduce the rendering to only look
    # for fields which are actually present
    real_fields = []
    for name, key in fields:
        if key in res:
            real_fields.append((name, key))

    formatted_print(res, text_format=FORMAT_TEXT_RECORD, fields=real_fields)
