from gcs_cli.helpers import filter_fields, requires_login, sorted_json_field
from gcs_cli.parsing import globus_command, sharing_policy_id_arg
from gcs_cli.prettyprint import FORMAT_TEXT_RECORD, formatted_print
from gcs_cli.services.gcs import get_client

STANDARD_FIELDS = [
    ("Sharing Policy ID", "id"),
    ("Collection ID", "collection_id"),
    ("Users", sorted_json_field("users")),
    ("Sharing Path Restrictions", sorted_json_field("sharing_restrict_paths")),
]


@globus_command("show", short_help="Show a Sharing Policy definition")
@sharing_policy_id_arg
@requires_login
def sharing_policy_show(sharing_policy_id):
    """
    Show an Endpoint Role definition
    """
    client = get_client()

    res = client.get_sharing_policy(sharing_policy_id)
    fields = filter_fields(STANDARD_FIELDS, res)

    formatted_print(res, text_format=FORMAT_TEXT_RECORD, fields=fields)
