import functools

from gcs_cli.helpers import record_print_result_message, requires_login
from gcs_cli.parsing import EXPLICIT_NULL
from gcs_cli.prettyprint import FORMAT_TEXT_RECORD, formatted_print
from gcs_cli.services.gcs import get_client


def make_create_command(func):
    @functools.wraps(func)
    @requires_login
    def wrapped(*args, **kwargs):
        sg_doc = func(*args, **kwargs)
        doc = {}
        for k, v in sg_doc.items():
            if v == EXPLICIT_NULL or v == (EXPLICIT_NULL,):
                doc[k] = None
            elif v is not None:
                doc[k] = v
        client = get_client()
        res = client.create_storage_gateway(doc)
        formatted_print(
            res, fields=[("Storage Gateway ID", "id")], text_format=FORMAT_TEXT_RECORD
        )

    return wrapped


def make_update_command(func):
    @functools.wraps(func)
    @requires_login
    def wrapped(*args, **kwargs):
        storage_gateway_id, sg_doc = func(*args, **kwargs)
        doc = {}
        for k, v in sg_doc.items():
            if v == EXPLICIT_NULL or v == (EXPLICIT_NULL,):
                doc[k] = None
            elif v is not None:
                doc[k] = v

        client = get_client()
        res = client.update_storage_gateway(storage_gateway_id, doc)
        record_print_result_message(res)

    return wrapped
