from gcs_cli.commands.storage_gateway._common import make_create_command
from gcs_cli.parsing import (
    globus_command,
    google_drive_options,
    storage_gateway_create_and_update_params,
)
from gcs_cli.sdk_client.data import GoogleDriveStoragePolicies, StorageGatewayDocument
from gcs_cli.services.gcs import GOOGLE_DRIVE_CONNECTOR_ID


@globus_command("google-drive")
@storage_gateway_create_and_update_params(create=True)
@google_drive_options
@make_create_command
def google_drive_create(
    *,
    google_client_id,
    google_client_secret,
    google_drive_user_api_rate_quota,
    **kwargs
):
    """Create a new Google Drive Storage Gateway"""
    policies = GoogleDriveStoragePolicies(
        client_id=google_client_id,
        secret=google_client_secret,
        user_api_rate_quota=google_drive_user_api_rate_quota,
    )
    return StorageGatewayDocument(
        connector_id=GOOGLE_DRIVE_CONNECTOR_ID, policies=policies, **kwargs
    )
