from gcs_cli.commands.storage_gateway._common import make_create_command
from gcs_cli.parsing import (
    globus_command,
    irods_options,
    storage_gateway_create_and_update_params,
)
from gcs_cli.sdk_client.data import IrodsStoragePolicies, StorageGatewayDocument
from gcs_cli.services.gcs import IRODS_CONNECTOR_ID


@globus_command("irods", short_help="Create a new iRODS storage gateway")
@storage_gateway_create_and_update_params(create=True)
@irods_options
@make_create_command
def irods_create(*, irods_environment_file, irods_authentication_file, **kwargs):
    """
    Create a new storage gateway. Requires a display name and a connector.
    Additional connector specific options may be required as well.
    """
    policies = None
    if irods_environment_file is not None or irods_authentication_file is not None:
        policies = IrodsStoragePolicies(
            irods_authentication_file=irods_authentication_file,
            irods_environment_file=irods_environment_file,
        )

    return StorageGatewayDocument(
        connector_id=IRODS_CONNECTOR_ID, policies=policies, **kwargs
    )
