from gcs_cli.commands.storage_gateway._common import make_update_command
from gcs_cli.parsing import (
    EXPLICIT_NULL,
    azure_blob_options,
    globus_command,
    storage_gateway_create_and_update_params,
    storage_gateway_id_arg,
)
from gcs_cli.sdk_client.data import AzureBlobStoragePolicies, StorageGatewayDocument
from gcs_cli.services.gcs import AZURE_BLOB_CONNECTOR_ID


@globus_command("azure-blob", short_help="Update an Azure Blob Storage Gateway")
@storage_gateway_id_arg
@storage_gateway_create_and_update_params(create=False)
@azure_blob_options
@make_update_command
def azure_blob_update(
    storage_gateway_id,
    *,
    ms_client_id,
    ms_client_secret,
    ms_tenant,
    azure_storage_account,
    azure_credential_type,
    azure_adls,
    **kwargs
):
    """
    Update an existing Azure Blob Storage Gateway.
    Any options given will replace existing values in the storage gateway's definition.
    Any options not given will retain their existing values.
    """
    policies = None
    if any(
        x is not None
        for x in (
            ms_client_id,
            ms_client_secret,
            ms_tenant,
            azure_storage_account,
            azure_credential_type,
            azure_adls,
        )
    ):
        policies = AzureBlobStoragePolicies(
            client_id=ms_client_id,
            secret=ms_client_secret,
            tenant=ms_tenant,
            account=azure_storage_account,
            auth_type=azure_credential_type,
            adls=azure_adls,
        )

    if ms_tenant == EXPLICIT_NULL:
        policies["tenant"] = None

    return (
        storage_gateway_id,
        StorageGatewayDocument(
            connector_id=AZURE_BLOB_CONNECTOR_ID, policies=policies, **kwargs
        ),
    )
