from gcs_cli.commands.storage_gateway._common import make_update_command
from gcs_cli.parsing import (
    EXPLICIT_NULL,
    globus_command,
    onedrive_options,
    storage_gateway_create_and_update_params,
    storage_gateway_id_arg,
)
from gcs_cli.sdk_client.data import OneDriveStoragePolicies, StorageGatewayDocument
from gcs_cli.services.gcs import ONEDRIVE_CONNECTOR_ID


@globus_command("onedrive", short_help="Update a OneDrive Storage Gateway")
@storage_gateway_id_arg
@storage_gateway_create_and_update_params(create=False)
@onedrive_options
@make_update_command
def onedrive_update(
    storage_gateway_id,
    *,
    ms_client_id,
    ms_client_secret,
    ms_tenant,
    ms_user_api_rate_limit,
    **kwargs
):
    """
    Update an existing OneDrive Storage Gateway.
    Any options given will replace existing values in the storage gateway's definition.
    Any options not given will retain their existing values.
    """
    policies = None
    if any(
        x is not None
        for x in (
            ms_client_id,
            ms_client_secret,
            ms_tenant,
            ms_user_api_rate_limit,
        )
    ):
        policies = OneDriveStoragePolicies(
            client_id=ms_client_id,
            secret=ms_client_secret,
            tenant=ms_tenant,
            user_api_rate_limit=ms_user_api_rate_limit,
        )

    if ms_tenant == EXPLICIT_NULL:
        policies["tenant"] = None

    return (
        storage_gateway_id,
        StorageGatewayDocument(
            connector_id=ONEDRIVE_CONNECTOR_ID, policies=policies, **kwargs
        ),
    )
