from globus_sdk.authorizers import RefreshTokenAuthorizer

from gcs_cli import version
from gcs_cli.config import (
    GCS_CLIENT_DATA_CONFIG_KEY,
    internal_auth_client,
    token_storage_adapter,
)
from gcs_cli.parsing.stateful_options import get_explicit_host
from gcs_cli.sdk_client import GCSClient

POSIX_CONNECTOR_ID = "145812c8-decc-41f1-83cf-bb2a85a2a70b"
POSIX_DISPLAY_NAME = "POSIX"

AZURE_BLOB_CONNECTOR_ID = "9436da0c-a444-11eb-af93-12704e0d6a4d"
AZURE_BLOB_DISPLAY_NAME = "Azure Blob"

BLACKPEARL_CONNECTOR_ID = "7e3f3f5e-350c-4717-891a-2f451c24b0d4"
BLACKPEARL_DISPLAY_NAME = "BlackPearl"

BOX_CONNECTOR_ID = "7c100eae-40fe-11e9-95a3-9cb6d0d9fd63"
BOX_DISPLAY_NAME = "Box"

CEPH_CONNECTOR_ID = "1b6374b0-f6a4-4cf7-a26f-f262d9c6ca72"
CEPH_DISPLAY_NAME = "Ceph"

ONEDRIVE_CONNECTOR_ID = "28ef55da-1f97-11eb-bdfd-12704e0d6a4d"
ONEDRIVE_DISPLAY_NAME = "OneDrive"

GOOGLE_DRIVE_CONNECTOR_ID = "976cf0cf-78c3-4aab-82d2-7c16adbcc281"
GOOGLE_DRIVE_DISPLAY_NAME = "Google Drive"

GOOGLE_CLOUD_STORAGE_CONNECTOR_ID = "56366b96-ac98-11e9-abac-9cb6d0d9fd63"
GOOGLE_CLOUD_STORAGE_DISPLAY_NAME = "Google Cloud Storage"

ACTIVESCALE_CONNECTOR_ID = "7251f6c8-93c9-11eb-95ba-12704e0d6a4d"
ACTIVESCALE_DISPLAY_NAME = "ActiveScale"

S3_CONNECTOR_ID = "7643e831-5f6c-4b47-a07f-8ee90f401d23"
S3_DISPLAY_NAME = "S3"

POSIX_STAGING_CONNECTOR_ID = "052be037-7dda-4d20-b163-3077314dc3e6"
POSIX_STAGING_DISPLAY_NAME = "POSIX Staging"

IRODS_CONNECTOR_ID = "e47b6920-ff57-11ea-8aaa-000c297ab3c2"
IRODS_DISPLAY_NAME = "iRODS"


def get_client():
    adapter = token_storage_adapter()
    gcs_client_data = adapter.read_config(GCS_CLIENT_DATA_CONFIG_KEY)
    token_data = adapter.read_as_dict().get(gcs_client_data["client_id"])
    authorizer = None

    # if there's token data, use it to build the authorizer
    if token_data is not None:
        authorizer = RefreshTokenAuthorizer(
            token_data["refresh_token"],
            internal_auth_client(),
            token_data["access_token"],
            token_data["expires_at_seconds"],
            on_refresh=adapter.on_refresh,
        )

    client = GCSClient(
        gcs_client_data["fqdn"], authorizer=authorizer, app_name=version.app_name
    )
    explicit_host = get_explicit_host()
    if explicit_host:
        client.set_address(explicit_host)
    return client


def connector_id_to_display_name(connector_id):
    mapping = {
        POSIX_CONNECTOR_ID: POSIX_DISPLAY_NAME,
        AZURE_BLOB_CONNECTOR_ID: AZURE_BLOB_DISPLAY_NAME,
        BLACKPEARL_CONNECTOR_ID: BLACKPEARL_DISPLAY_NAME,
        BOX_CONNECTOR_ID: BOX_DISPLAY_NAME,
        CEPH_CONNECTOR_ID: CEPH_DISPLAY_NAME,
        GOOGLE_DRIVE_CONNECTOR_ID: GOOGLE_DRIVE_DISPLAY_NAME,
        GOOGLE_CLOUD_STORAGE_CONNECTOR_ID: GOOGLE_CLOUD_STORAGE_DISPLAY_NAME,
        ONEDRIVE_CONNECTOR_ID: ONEDRIVE_DISPLAY_NAME,
        S3_CONNECTOR_ID: S3_DISPLAY_NAME,
        ACTIVESCALE_CONNECTOR_ID: ACTIVESCALE_DISPLAY_NAME,
        POSIX_STAGING_CONNECTOR_ID: POSIX_STAGING_DISPLAY_NAME,
        IRODS_CONNECTOR_ID: IRODS_DISPLAY_NAME,
    }
    display_name = mapping.get(connector_id)

    if not display_name:
        display_name = "UNKNOWN ({})".format(connector_id)

    return display_name


def connector_display_name_to_id(display_name):
    mapping = {
        POSIX_DISPLAY_NAME: POSIX_CONNECTOR_ID,
        AZURE_BLOB_DISPLAY_NAME: AZURE_BLOB_CONNECTOR_ID,
        BLACKPEARL_DISPLAY_NAME: BLACKPEARL_CONNECTOR_ID,
        BOX_DISPLAY_NAME: BOX_CONNECTOR_ID,
        CEPH_DISPLAY_NAME: CEPH_CONNECTOR_ID,
        GOOGLE_DRIVE_DISPLAY_NAME: GOOGLE_DRIVE_CONNECTOR_ID,
        GOOGLE_CLOUD_STORAGE_DISPLAY_NAME: GOOGLE_CLOUD_STORAGE_CONNECTOR_ID,
        ONEDRIVE_DISPLAY_NAME: ONEDRIVE_CONNECTOR_ID,
        S3_DISPLAY_NAME: S3_CONNECTOR_ID,
        ACTIVESCALE_DISPLAY_NAME: ACTIVESCALE_CONNECTOR_ID,
        POSIX_STAGING_DISPLAY_NAME: POSIX_STAGING_CONNECTOR_ID,
        IRODS_DISPLAY_NAME: IRODS_CONNECTOR_ID,
    }
    return mapping.get(display_name)
