from collections import namedtuple
from typing import Dict

GridFTPConfigOption = namedtuple("GridFTPConfigOption", ["name", "type", "default"])

options: Dict[str, GridFTPConfigOption] = {
    "help": GridFTPConfigOption(name="help", type="bool", default=False),
    "longhelp": GridFTPConfigOption(name="longhelp", type="bool", default=False),
    "version": GridFTPConfigOption(name="version", type="bool", default=False),
    "versions": GridFTPConfigOption(name="versions", type="bool", default=False),
    "inetd": GridFTPConfigOption(name="inetd", type="bool", default=False),
    "daemon": GridFTPConfigOption(name="daemon", type="bool", default=True),
    "detach": GridFTPConfigOption(name="detach", type="bool", default=False),
    "ssh": GridFTPConfigOption(name="ssh", type="bool", default=False),
    "exec": GridFTPConfigOption(name="exec", type="string", default=None),
    "chdir": GridFTPConfigOption(name="chdir", type="bool", default=True),
    "chdir_to": GridFTPConfigOption(name="chdir_to", type="string", default=None),
    "threads": GridFTPConfigOption(name="threads", type="int", default=0),
    "fork": GridFTPConfigOption(name="fork", type="bool", default=True),
    "fork_fallback": GridFTPConfigOption(
        name="fork_fallback", type="bool", default=False
    ),
    "single": GridFTPConfigOption(name="single", type="bool", default=False),
    "chroot_path": GridFTPConfigOption(name="chroot_path", type="string", default=None),
    "auth_level": GridFTPConfigOption(name="auth_level", type="int", default=3),
    "process_user": GridFTPConfigOption(
        name="process_user", type="string", default=None
    ),
    "process_group": GridFTPConfigOption(
        name="process_group", type="string", default=None
    ),
    "ipc_allow_from": GridFTPConfigOption(
        name="ipc_allow_from", type="string", default=None
    ),
    "debug": GridFTPConfigOption(name="debug", type="bool", default=False,),
    "ignore_bad_threads": GridFTPConfigOption(
        name="ignore_bad_threads", type="bool", default=False,
    ),
    "bad_signal_exit": GridFTPConfigOption(
        name="bad_signal_exit", type="bool", default=True,
    ),
    "ipc_deny_from": GridFTPConfigOption(
        name="ipc_deny_from", type="string", default=None
    ),
    "slow_dirlist": GridFTPConfigOption(name="slow_dirlist", type="int", default=0),
    "checksum_throttle": GridFTPConfigOption(
        name="checksum_throttle", type="int", default=0
    ),
    "globus_location": GridFTPConfigOption(
        name="globus_location", type="string", default=None
    ),
    "pidfile": GridFTPConfigOption(name="pidfile", type="string", default=None),
    "allow_from": GridFTPConfigOption(name="allow_from", type="string", default=None),
    "deny_from": GridFTPConfigOption(name="deny_from", type="string", default=None),
    "encrypt_data": GridFTPConfigOption(
        name="encrypt_data", type="bool", default=False
    ),
    "secure_ipc": GridFTPConfigOption(name="secure_ipc", type="bool", default=True),
    "ipc_auth_mode": GridFTPConfigOption(
        name="ipc_auth_mode", type="string", default="host"
    ),
    "ipc_user_name": GridFTPConfigOption(
        name="ipc_user_name", type="string", default=None
    ),
    "ipc_subject": GridFTPConfigOption(name="ipc_subject", type="string", default=None),
    "ipc_credential": GridFTPConfigOption(
        name="ipc_credential", type="string", default=None
    ),
    "ipc_cookie": GridFTPConfigOption(name="ipc_cookie", type="string", default=None),
    "allow_anonymous": GridFTPConfigOption(
        name="allow_anonymous", type="bool", default=False
    ),
    "anonymous_names_allowed": GridFTPConfigOption(
        name="anonymous_names_allowed", type="string", default=None
    ),
    "control_tls": GridFTPConfigOption(name="control_tls", type="bool", default=False),
    "anonymous_user": GridFTPConfigOption(
        name="anonymous_user", type="string", default=None
    ),
    "anonymous_group": GridFTPConfigOption(
        name="anonymous_group", type="string", default=None
    ),
    "sharing_dn": GridFTPConfigOption(name="sharing_dn", type="string", default=None),
    "sharing_state_dir": GridFTPConfigOption(
        name="sharing_state_dir", type="string", default=None
    ),
    "sharing_control": GridFTPConfigOption(
        name="sharing_control", type="bool", default=True
    ),
    "sharing_rp": GridFTPConfigOption(name="sharing_rp", type="string", default=None),
    "sharing_users_allow": GridFTPConfigOption(
        name="sharing_users_allow", type="string", default=None
    ),
    "sharing_users_deny": GridFTPConfigOption(
        name="sharing_users_deny", type="string", default=None
    ),
    "sharing_groups_allow": GridFTPConfigOption(
        name="sharing_groups_allow", type="string", default=None
    ),
    "sharing_groups_deny": GridFTPConfigOption(
        name="sharing_groups_deny", type="string", default=None
    ),
    "allow_root": GridFTPConfigOption(name="allow_root", type="bool", default=False),
    "allow_disabled_login": GridFTPConfigOption(
        name="allow_disabled_login", type="bool", default=False
    ),
    "pw_file": GridFTPConfigOption(name="pw_file", type="string", default=None),
    "connections_max": GridFTPConfigOption(
        name="connections_max", type="int", default=-1
    ),
    "connections_disabled": GridFTPConfigOption(
        name="connections_disabled", type="bool", default=False
    ),
    "offline_msg": GridFTPConfigOption(name="offline_msg", type="string", default=None),
    "disable_command_list": GridFTPConfigOption(
        name="disable_command_list", type="string", default=None
    ),
    "cas": GridFTPConfigOption(name="cas", type="bool", default=True),
    "use_home_dirs": GridFTPConfigOption(
        name="use_home_dirs", type="bool", default=True
    ),
    "home_dir": GridFTPConfigOption(name="home_dir", type="string", default=None),
    "default_dir": GridFTPConfigOption(name="default_dir", type="string", default=None),
    "restrict_paths": GridFTPConfigOption(
        name="restrict_paths", type="string", default=None
    ),
    "rp_follow_symlinks": GridFTPConfigOption(
        name="rp_follow_symlinks", type="bool", default=False
    ),
    "acl": GridFTPConfigOption(name="acl", type="string", default=None),
    "log_level": GridFTPConfigOption(name="log_level", type="string", default="ERROR"),
    "log_module": GridFTPConfigOption(name="log_module", type="string", default=None),
    "log_single": GridFTPConfigOption(name="log_single", type="string", default=None),
    "log_unique": GridFTPConfigOption(name="log_unique", type="string", default=None),
    "log_transfer": GridFTPConfigOption(
        name="log_transfer", type="string", default=None
    ),
    "log_audit": GridFTPConfigOption(name="log_audit", type="string", default=None),
    "log_audit_dated": GridFTPConfigOption(
        name="log_audit_dated", type="bool", default=False
    ),
    "log_audit_limit": GridFTPConfigOption(
        name="log_audit_limit", type="string", default=None
    ),
    "log_filemode": GridFTPConfigOption(
        name="log_filemode", type="string", default=None
    ),
    "disable_usage_stats": GridFTPConfigOption(
        name="disable_usage_stats", type="bool", default=False
    ),
    "usage_stats_target": GridFTPConfigOption(
        name="usage_stats_target", type="string", default=None
    ),
    "usage_stats_id": GridFTPConfigOption(
        name="usage_stats_id", type="string", default=None
    ),
    "remote_nodes": GridFTPConfigOption(
        name="remote_nodes", type="string", default=None
    ),
    "hybrid": GridFTPConfigOption(name="hybrid", type="bool", default=False),
    "data_node": GridFTPConfigOption(name="data_node", type="bool", default=False),
    "combined_data_node": GridFTPConfigOption(
        name="combined_data_node", type="bool", default=False
    ),
    "stripe_blocksize": GridFTPConfigOption(
        name="stripe_blocksize", type="int", default=1048576
    ),
    "stripe_count": GridFTPConfigOption(name="stripe_count", type="int", default=1),
    "brain": GridFTPConfigOption(name="brain", type="string", default=None),
    "stripe_layout": GridFTPConfigOption(name="stripe_layout", type="int", default=2),
    "stripe_blocksize_locked": GridFTPConfigOption(
        name="stripe_blocksize_locked", type="bool", default=False
    ),
    "stripe_layout_locked": GridFTPConfigOption(
        name="stripe_layout_locked", type="bool", default=False
    ),
    "stripe_mode": GridFTPConfigOption(name="stripe_mode", type="int", default=1),
    "blocksize": GridFTPConfigOption(name="blocksize", type="int", default=262144),
    "sync_writes": GridFTPConfigOption(name="sync_writes", type="bool", default=False),
    "direct": GridFTPConfigOption(name="direct", type="bool", default=False),
    "perms": GridFTPConfigOption(name="perms", type="string", default=None),
    "file_timeout": GridFTPConfigOption(name="file_timeout", type="int", default=0),
    "port": GridFTPConfigOption(name="port", type="int", default=0),
    "control_interface": GridFTPConfigOption(
        name="control_interface", type="string", default=None
    ),
    "data_interface": GridFTPConfigOption(
        name="data_interface", type="string", default=None
    ),
    "ipc_interface": GridFTPConfigOption(
        name="ipc_interface", type="string", default=None
    ),
    "hostname": GridFTPConfigOption(name="hostname", type="string", default=None),
    "ipc_port": GridFTPConfigOption(name="ipc_port", type="int", default=0),
    "ipc_uds_path": GridFTPConfigOption(
        name="ipc_uds_path", type="string", default=None
    ),
    "uds_owner": GridFTPConfigOption(name="uds_owner", type="string", default=None),
    "control_preauth_timeout": GridFTPConfigOption(
        name="control_preauth_timeout", type="int", default=120
    ),
    "control_idle_timeout": GridFTPConfigOption(
        name="control_idle_timeout", type="int", default=600
    ),
    "ipc_idle_timeout": GridFTPConfigOption(
        name="ipc_idle_timeout", type="int", default=900
    ),
    "ipc_connect_timeout": GridFTPConfigOption(
        name="ipc_connect_timeout", type="int", default=60
    ),
    "always_send_markers": GridFTPConfigOption(
        name="always_send_markers", type="bool", default=False
    ),
    "allow_udt": GridFTPConfigOption(name="allow_udt", type="bool", default=False),
    "port_range": GridFTPConfigOption(name="port_range", type="string", default=None),
    "banner": GridFTPConfigOption(name="banner", type="string", default=None),
    "banner_file": GridFTPConfigOption(name="banner_file", type="string", default=None),
    "banner_terse": GridFTPConfigOption(
        name="banner_terse", type="bool", default=False
    ),
    "banner_append": GridFTPConfigOption(
        name="banner_append", type="bool", default=False
    ),
    "version_tag": GridFTPConfigOption(name="version_tag", type="string", default=None),
    "login_msg": GridFTPConfigOption(name="login_msg", type="string", default=None),
    "login_msg_file": GridFTPConfigOption(
        name="login_msg_file", type="string", default=None
    ),
    "load_dsi_module": GridFTPConfigOption(
        name="load_dsi_module", type="string", default=None
    ),
    "allowed_modules": GridFTPConfigOption(
        name="allowed_modules", type="string", default=None
    ),
    "dc_whitelist": GridFTPConfigOption(
        name="dc_whitelist", type="string", default=None
    ),
    "fs_whitelist": GridFTPConfigOption(
        name="fs_whitelist", type="string", default=None
    ),
    "popen_whitelist": GridFTPConfigOption(
        name="popen_whitelist", type="string", default=None
    ),
    "xnetmgr": GridFTPConfigOption(name="xnetmgr", type="string", default=None),
    "dc_default": GridFTPConfigOption(name="dc_default", type="string", default=None),
    "fs_default": GridFTPConfigOption(name="fs_default", type="string", default=None),
    "$GSI_AUTHZ_CONF": GridFTPConfigOption(
        name="$GSI_AUTHZ_CONF", type="string", default=None
    ),
    "$GRIDMAP": GridFTPConfigOption(name="$GRIDMAP", type="string", default=None),
    "$X509_USER_CERT": GridFTPConfigOption(
        name="$X509_USER_CERT", type="string", default=None
    ),
    "$X509_USER_KEY": GridFTPConfigOption(
        name="$X509_USER_KEY", type="string", default=None
    ),
    "$X509_CERT_DIR": GridFTPConfigOption(
        name="$X509_CERT_DIR", type="string", default=None
    ),
    "$GLOBUS_TCP_SOURCE_RANGE": GridFTPConfigOption(
        name="$GLOBUS_TCP_SOURCE_RANGE", type="string", default=None
    ),
    "$GLOBUS_MYPROXY_CA_CERT": GridFTPConfigOption(
        name="$GLOBUS_MYPROXY_CA_CERT", type="string", default=None
    ),
    # FIXME: Are there other environment variables we'd want to parse?
}
