import os
import re
from typing import Dict, List, Optional, Union


class Gridmap:
    def __init__(
        self,
        path: Union[os.PathLike, str] = None,
        mappings: Optional[Dict[str, List[str]]] = None,
    ):
        self.path = path
        self.mappings = mappings

    @property
    def mappings(self) -> Optional[Dict[str, List[str]]]:
        if not self._mappings:
            mappings = {}
            if self.path and os.path.exists(self.path):
                with open(self.path, "r") as f:
                    for line in f:
                        line = re.sub("#.*", "", line)
                        line = re.sub(r"^\s*", "", line)
                        if not line:
                            continue
                        m = re.match(
                            r'^(?P<dn>[^"]\S+|(["]([^"]|[\\].)*["]))\s+'
                            r"(?P<local>([^\s,]+(\s*,\s*[^\s,]+)*))\s*$",
                            line,
                        )
                        if m:
                            dn = m.group("dn")
                            if dn.startswith('"'):
                                dn = dn[1:-1]
                                if '\\"' in dn:
                                    dn = re.sub(r'\\"', '"', dn)
                            local = m.group("local").split(",")
                            if dn not in mappings:
                                mappings[dn] = [lo.strip() for lo in local]
                            else:
                                mappings[dn].extend([lo.strip() for lo in local])

            self.mappings = mappings
        return self._mappings

    @mappings.setter
    def mappings(self, value: Dict[str, List[str]]) -> None:
        self._mappings = value
