# pragma: no cover
from typing import List, Optional, Union
from typing_extensions import Literal, TypedDict
from .transfer_documents import NetworkUseName, PrincipalTypeName, ACLPermissionName


class _ExpressionIdentityMappingEntryDict(TypedDict, total=False):
    source: str
    match: str
    output: str
    literal: bool


class ExpressionIdentityMappingDict(TypedDict, total=False):
    DATA_TYPE: Literal["expression_identity_mapping#1.0.0"]
    mappings: List[_ExpressionIdentityMappingEntryDict]


class ExternalIdentityMappingDict(TypedDict, total=False):
    DATA_TYPE: Literal["external_identity_mapping#1.0.0"]
    command: List[str]


IdentityMappingDict = Union[ExpressionIdentityMappingDict, ExternalIdentityMappingDict]


class ManagedDomainDict(TypedDict, total=False):
    DATA_TYPE: Literal["domain#1.0.0"]
    domain_name: str
    wildcard: bool
    certificate: str
    certificate_chain: Optional[str]
    private_key: str


class UnmanagedDomainDict(TypedDict, total=False):
    DATA_TYPE: Literal["domain#1.0.0"]
    domain_name: str
    wildcard: bool
    certificate_path: str
    certificate_chain_path: Optional[str]
    private_key_path: str


DomainDict = Union[ManagedDomainDict, UnmanagedDomainDict]


class PathRestrictionsDict(TypedDict):
    DATA_TYPE: Literal["path_restrictions#1.0.0"]
    read: List[str]
    read_write: List[str]
    none: List[str]


class SharingPoliciesDict(TypedDict, total=False):
    DATA_TYPE: Literal["sharing_policy#1.0.0"]
    id: str
    collection_id: str
    sharing_restrict_paths: PathRestrictionsDict
    users: Optional[List[str]]


class PosixStorageGatewayPoliciesDict(TypedDict, total=False):
    DATA_TYPE: Literal["posix_storage_policies#1.0.0"]
    groups_allow: Optional[List[str]]
    groups_deny: Optional[List[str]]


class PosixStorageGatewayDict(TypedDict, total=False):
    DATA_TYPE: Literal["storage_gateway#1.1.0"]
    id: str
    display_name: str
    connector_id: Literal["145812c8-decc-41f1-83cf-bb2a85a2a70b"]
    high_assurance: bool
    authentication_timeout_mins: int
    allowed_domains: List[str]
    identity_mappings: Optional[List[IdentityMappingDict]]
    users_allow: Optional[List[str]]
    users_deny: Optional[List[str]]
    restrict_paths: Optional[PathRestrictionsDict]
    require_mfa: bool
    policies: Optional[PosixStorageGatewayPoliciesDict]


class _MetadataDict(TypedDict, total=False):
    display_name: str
    organization: str
    department: str
    keywords: List[str]
    description: str
    contact_email: str
    contact_info: str
    info_link: str


class EndpointDict(_MetadataDict):
    DATA_TYPE: Literal["endpoint#1.0.0"]
    public: bool
    subscription_id: Optional[str]
    network_use: Optional[NetworkUseName]
    max_concurrency: Optional[int]
    preferred_concurrency: Optional[int]
    max_parallelism: Optional[int]
    preferred_parallelism: Optional[int]
    allow_udt: bool


class PosixCollectionPoliciesDict(TypedDict, total=False):
    DATA_TYPE: Literal["posix_collection_policies#1.1.0"]
    sharing_groups_allow: List[str]
    sharing_groups_deny: List[str]


class PosixMappedCollectionDict(_MetadataDict):
    DATA_TYPE: Literal["collection#1.4.0"]
    domain_name: str
    collection_type: Literal["mapped"]
    identity_id: str
    storage_gateway_id: str
    collection_base_path: str
    default_directory: str
    public: bool
    force_encryption: bool
    disable_verify: bool
    high_assurance: bool
    authentication_timeout_mins: int
    allow_guest_collections: bool
    sharing_restrict_paths: Optional[PathRestrictionsDict]

    policies: PosixCollectionPoliciesDict

    # 1.1.0 policies
    enable_https: bool
    user_message: str
    user_message_link: str

    # 1.2.0 policies
    sharing_users_allow: Optional[List[str]]
    sharing_users_deny: Optional[List[str]]

    # 1.3.0 policies
    domain: DomainDict

    # 1.4.0 policies
    force_verify: bool
    require_mfa: bool


RoleName = Union[
    Literal["owner"],
    Literal["administrator"],
    Literal["access_manager"],
    Literal["activity_manager"],
    Literal["activity_monitor"],
    Literal["access_monitor"],
]


class RoleDict(TypedDict, total=False):
    DATA_TYPE: Literal["role#1.0.0"]
    id: str
    principal: str
    collection: Optional[str]
    role: RoleName


class ACLDict(TypedDict):
    DATA_TYPE: Literal["access"]
    role_id: Literal[None]
    role_type: Literal[None]
    principal_type: Optional[PrincipalTypeName]
    principal: Optional[str]
    path: Optional[str]
    permissions: Optional[ACLPermissionName]


class PosixUserCredentialPoliciesDict(TypedDict, total=False):
    DATA_TYPE: Literal["posix_user_credential_policies#1.0.0"]


class PosixUserCredentialDict(TypedDict, total=False):
    DATA_TYPE: Literal["user_credential#1.0.0"]
    id: str
    identity_id: str
    connector_id: Literal["145812c8-decc-41f1-83cf-bb2a85a2a70b"]
    storage_gateway_id: str
    username: str
    display_name: str
    invalid: bool
    provisioned: bool
    deleted: bool
    policies: PosixUserCredentialPoliciesDict


class PosixGuestCollectionDict(_MetadataDict):
    DATA_TYPE: Literal["collection#1.4.0"]
    domain_name: str
    collection_type: Literal["guest"]
    identity_id: str
    storage_gateway_id: str
    mapped_collection_id: str
    collection_base_path: str
    default_directory: str
    public: bool
    force_encryption: bool
    disable_verify: bool
    high_assurance: bool
    authentication_timeout_mins: int

    policies: PosixCollectionPoliciesDict

    # 1.1.0 policies
    enable_https: bool
    user_message: str
    user_message_link: str

    # 1.3.0 policies
    domain: DomainDict

    # 1.4.0 policies
    force_verify: bool
    require_mfa: bool
