from typing import Optional, Union

import globus_sdk

from ..config import internal_auth_client, token_storage_adapter


def get_api(
    service: Optional[str] = None,
) -> Union[globus_sdk.TransferClient, globus_sdk.AuthClient]:
    adapter = token_storage_adapter()

    token_data = adapter.read_as_dict().get(service or "transfer.api.globus.org")
    authorizer = None

    if service is None:
        service = "transfer.api.globus.org"

    # if there's token data, use it to build the authorizer
    if token_data is not None:
        authorizer = globus_sdk.RefreshTokenAuthorizer(
            token_data["refresh_token"],
            internal_auth_client(),
            token_data["access_token"],
            token_data["expires_at_seconds"],
            on_refresh=adapter.on_refresh,
        )

    if service == "transfer.api.globus.org":
        return globus_sdk.TransferClient(authorizer=authorizer)
    elif service == "auth.globus.org":
        return globus_sdk.AuthClient(authorizer=authorizer)
    raise Exception("Unsupported service")
