from flask import Blueprint, current_app, g, jsonify

from globus.manager.api.models.result import Result
from globus.manager.api.schemas.api import (
    connector_schema,
    info_schema,
    result_schema,
)
from globus.manager.api.utils.info import info
from globus.manager.api.blueprints.decorators import (
    Base62Marker,
    paginated,
)


blueprint = Blueprint("info_blueprint", __name__, url_prefix="/api")


@blueprint.route("/info", methods=["GET"])
@paginated
def info_get():
    """
    GET /info

    Returns information about this GCS Manager such as the API version.
    """
    marker = g.marker
    page_size = g.page_size
    next_page_marker = None
    data = []

    if not marker:
        data.append(info_schema.dump(info()))

    connectors = [
        current_app.config["connectors"][k]
        for k in sorted(current_app.config["connectors"].keys())
        if k >= (marker.marker if marker else "")
    ][0 : page_size + 1]

    for conn in connectors:
        if len(data) == page_size:
            next_page_marker = Base62Marker(marker_data=conn.CONNECTOR_ID)
            break

        connector_info = conn.model.connector

        if connector_info.enabled:
            data.append(connector_schema.dump(connector_info))

    res = result_schema.dump(Result(data=data, marker=next_page_marker))

    return jsonify(res)
