from typing import List

import marshmallow as ma
from mypy_extensions import TypedDict
from typing_extensions import Literal

from .data_type import DataTypeSchema
from .schema_version_checker import verify_schema_version

_PATH_RE = "^[/~\\$\\*\\?\\[].*$"

PathRestrictionsDict = TypedDict(
    "PathRestrictionsDict",
    {
        "DATA_TYPE": Literal["path_restrictions#1.0.0"],
        "read": List[str],
        "read_write": List[str],
        "none": List[str],
    },
)


class PathRestrictionsSchema(ma.Schema):
    _DATA_TYPE = "path_restrictions#1.0.0"
    DATA_TYPE = ma.fields.String(required=True, allow_none=False, default=_DATA_TYPE)

    read = ma.fields.List(ma.fields.String(validate=[ma.validate.Regexp(_PATH_RE)]))
    read_write = ma.fields.List(
        ma.fields.String(validate=[ma.validate.Regexp(_PATH_RE)])
    )
    none = ma.fields.List(ma.fields.String(validate=[ma.validate.Regexp(_PATH_RE)]))

    def load(self, *args, **kwargs) -> PathRestrictionsDict:
        return super().load(*args, **kwargs)

    @ma.validates("DATA_TYPE")
    def validate_data_type(self, data, **kwargs):
        verify_schema_version(data, self._DATA_TYPE)


path_restrictions_schema = PathRestrictionsSchema()
DataTypeSchema.register_schema(path_restrictions_schema)
