from typing import Union
import uuid

import marshmallow as ma
from mypy_extensions import TypedDict
from typing_extensions import Literal

from globus.manager.constants import PrincipalType, RoleType
from globus.manager.api.schemas.enum_field import EnumField


RoleConfigDict = TypedDict(
    "RoleConfigDict",
    {
        "DATA_TYPE": Literal["role_config#1.0.0"],
        "id": uuid.UUID,
        "principal_type": PrincipalType,
        "identity_id": uuid.UUID,
        "role": RoleType,
        "resource": Union[None, uuid.UUID],
        "primary_role": bool,
    },
    total=False,
)


class RoleConfig(ma.Schema):
    _DATA_TYPE = "role_config#1.0.0"
    DATA_TYPE = ma.fields.Str(
        default=_DATA_TYPE, validate=[ma.validate.Equal(_DATA_TYPE)]
    )

    id = ma.fields.UUID()
    principal_type = EnumField(enum_class=PrincipalType)
    identity_id = ma.fields.UUID()
    role = EnumField(enum_class=RoleType)
    resource = ma.fields.UUID(allow_none=True)
    primary_role = ma.fields.Boolean()

    def load(self, *args, **kwargs) -> RoleConfigDict:
        return super().load(*args, **kwargs)


role_config_schema = RoleConfig()
