import os
import click

from globus_gcs.service_manager import ServiceManager
from gcs_cli.parsing.command_decorator import globus_command


@globus_command(
    name="cleanup",
    short_help="Cleanup the Globus OIDC server on this node",
    disable_options=["format", "explicit_host"],
    hidden=True,
    help="""\
The OIDC cleanup command disables the OIDC server on the GCS node.
This is automatically run during node cleanup.

== EXAMPLE

globus-connect-server oidc cleanup

""",
)
@click.pass_context
def oidc_cleanup(ctx):
    if os.getuid() != 0:
        click.echo("This command must be run as the root user", err=True)
        ctx.exit(1)

    cleanup = OIDCCleanup()
    cleanup.run()


class OIDCCleanup:
    def __init__(self):
        pass

    def run(self):
        tasks = [self.disable_service]

        def show(obj):
            if obj and obj.__doc__:
                return obj.__doc__.strip().split("\n")[0]
            return ""

        with click.progressbar(tasks, item_show_func=show, show_eta=False) as bar:
            for task in bar:
                task()

    def disable_service(self):
        """
        Disable the OIDC service
        """
        ServiceManager.disable("globus-oidc", path=True)
        ServiceManager.stop("globus-oidc")
