"""terminology rename

Revision ID: 1edd71022809
Revises: 6cb1ff7ce4c1
Create Date: 2018-03-01 21:54:59.735995

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '1edd71022809'
down_revision = '6cb1ff7ce4c1'
branch_labels = None
depends_on = None

"""
Note SQLite does not support direct re-naming of columns so the
following code must re-create then delete tables to rename their columns
"""


def upgrade():
    op.execute("PRAGMA foreign_keys = OFF")
    op.execute("PRAGMA legacy_alter_table=ON")
    # rename storage_type to connector
    op.rename_table('storage_type', 'connector')

    # rename storage_system to storage_gateway
    # and rename column storage_type_id to connector_id
    op.create_table(
        'storage_gateway',
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('display_name', sa.TEXT(), nullable=False),
        sa.Column('credential', sa.TEXT(), nullable=True),
        sa.Column('storage_policies', sa.TEXT(), nullable=True),
        sa.Column('deleted', sa.Boolean(), server_default='0', nullable=False),
        sa.Column('connector_id', sa.TEXT(), nullable=False),
        sa.ForeignKeyConstraint(['connector_id'], ['connector.id']),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('display_name'))
    op.execute("""
        INSERT INTO storage_gateway(
            created_at, updated_at, id, display_name, credential,
            storage_policies, deleted, connector_id)
        SELECT
            created_at, updated_at, id, display_name, credential,
            storage_policies, deleted, storage_type_id
        FROM storage_system;""")

    # rename columns storage_system_id and storage_system_user_id
    # to storage_gateway_id and storage_gateway_user_id
    # in user_credential table
    op.rename_table('user_credential', 'user_credential_old')
    op.create_table(
        'user_credential',
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('display_name', sa.TEXT(), nullable=True),
        sa.Column('identity_id', sa.TEXT(), nullable=False),
        sa.Column('storage_credential', sa.TEXT(), nullable=True),
        sa.Column('storage_gateway_id', sa.TEXT(), nullable=False),
        sa.Column('storage_gateway_user_id', sa.TEXT(), nullable=False),
        sa.Column('deleted', sa.Integer(), server_default='0', nullable=False),
        sa.Column('invalid', sa.Integer(), server_default='0', nullable=False),
        sa.UniqueConstraint('storage_gateway_id', 'storage_gateway_user_id',
                            'identity_id'),
        sa.ForeignKeyConstraint(['storage_gateway_id'],
                                ['storage_gateway.id']),
        sa.PrimaryKeyConstraint('id')
    )
    op.execute("""
        INSERT INTO user_credential(
            created_at, updated_at, id, display_name, identity_id,
            storage_credential, storage_gateway_id, storage_gateway_user_id,
            deleted, invalid)
        SELECT
            created_at, updated_at, id, display_name, identity_id,
            storage_credential, storage_system_id, storage_system_user_id,
            deleted, invalid
        FROM user_credential_old;""")

    # fix foreign key in collection to user_credential_old
    op.rename_table('collection', 'collection_old')
    op.create_table('collection',
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('path', sa.TEXT(), nullable=True),
        sa.Column('user_credential_id', sa.TEXT(), nullable=False),
        sa.Column('domain', sa.TEXT(), nullable=True),
        sa.ForeignKeyConstraint(['user_credential_id'],
                                ['user_credential.id']),
        sa.PrimaryKeyConstraint('id')
    )
    op.execute("""
        INSERT INTO collection(
            created_at, updated_at, id, path, user_credential_id, domain)
        SELECT
            created_at, updated_at, id, path, user_credential_id, domain
        FROM collection_old;""")

    # drop old tables without violating foreign key constraints
    op.drop_table('collection_old')
    op.drop_table('user_credential_old')
    op.drop_table('storage_system')

    # re-create indexes deleted during renaming
    op.create_index('ix_storage_gateway_deleted',
                    'storage_gateway', ['deleted'])
    op.create_index('ix_user_credential_storage_gateway_user_id',
                  'user_credential', ['storage_gateway_user_id'])
    op.create_index('ix_user_credential_deleted',
                  'user_credential', ['deleted'])
    op.create_index('ix_user_credential_invalid',
                  'user_credential', ['invalid'])
    op.create_index('ix_user_credential_identity_id',
                  'user_credential', ['identity_id'])


def downgrade():
    op.execute("PRAGMA foreign_keys = OFF")
    # rename connector to storage_type
    op.rename_table('connector', 'storage_type')

    # rename storage_gateway to storage_system
    # and rename column connector_id to storage_type_id
    op.create_table(
        'storage_system',
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('display_name', sa.TEXT(), nullable=False),
        sa.Column('credential', sa.TEXT(), nullable=True),
        sa.Column('storage_policies', sa.TEXT(), nullable=True),
        sa.Column('deleted', sa.Boolean(), server_default='0', nullable=False),
        sa.Column('storage_type_id', sa.TEXT(), nullable=False),
        sa.ForeignKeyConstraint(['storage_type_id'], ['storage_type.id']),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('display_name'))
    op.execute("""
        INSERT INTO storage_system(
            created_at, updated_at, id, display_name, credential,
            storage_policies, deleted, storage_type_id)
        SELECT
            created_at, updated_at, id, display_name, credential,
            storage_policies, deleted, connector_id
        FROM storage_gateway;""")

    # rename columns storage_gateway_id and storage_gateway_user_id
    # to storage_system_id and storage_system_user_id
    # in user_credential table
    op.rename_table('user_credential', 'user_credential_old')
    op.create_table(
        'user_credential',
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('display_name', sa.TEXT(), nullable=True),
        sa.Column('identity_id', sa.TEXT(), nullable=False),
        sa.Column('storage_credential', sa.TEXT(), nullable=True),
        sa.Column('storage_system_id', sa.TEXT(), nullable=False),
        sa.Column('storage_system_user_id', sa.TEXT(), nullable=False),
        sa.Column('deleted', sa.Integer(), server_default='0', nullable=False),
        sa.Column('invalid', sa.Integer(), server_default='0', nullable=False),
        sa.UniqueConstraint('storage_system_id', 'storage_system_user_id',
                            'identity_id'),
        sa.ForeignKeyConstraint(['storage_system_id'],
                                ['storage_system.id']),
        sa.PrimaryKeyConstraint('id')
    )
    op.execute("""
        INSERT INTO user_credential(
            created_at, updated_at, id, display_name, identity_id,
            storage_credential, storage_system_id, storage_system_user_id,
            deleted, invalid)
        SELECT
            created_at, updated_at, id, display_name, identity_id,
            storage_credential, storage_gateway_id, storage_gateway_user_id,
            deleted, invalid
        FROM user_credential_old;""")

    # fix foreign key in collection to now deleted user_credential_old
    op.rename_table('collection', 'collection_old')
    op.create_table('collection',
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('path', sa.TEXT(), nullable=True),
        sa.Column('user_credential_id', sa.TEXT(), nullable=False),
        sa.Column('domain', sa.TEXT(), nullable=True),
        sa.ForeignKeyConstraint(['user_credential_id'],
                                ['user_credential.id']),
        sa.PrimaryKeyConstraint('id')
    )
    op.execute("""
        INSERT INTO collection(
            created_at, updated_at, id, path, user_credential_id, domain)
        SELECT
            created_at, updated_at, id, path, user_credential_id, domain
        FROM collection_old;""")

    # drop old tables without violating foreign key constraints
    op.drop_table('collection_old')
    op.drop_table('user_credential_old')
    op.drop_table('storage_gateway')

    # re-create indexes deleted during renaming
    op.create_index('ix_storage_system_deleted',
                    'storage_system', ['deleted'])
    op.create_index('ix_user_credential_storage_system_user_id',
                  'user_credential', ['storage_system_user_id'])
    op.create_index('ix_user_credential_deleted',
                  'user_credential', ['deleted'])
    op.create_index('ix_user_credential_invalid',
                  'user_credential', ['invalid'])
    op.create_index('ix_user_credential_identity_id',
                  'user_credential', ['identity_id'])
