"""remove redundant deleted columns

Revision ID: 5a000ca9abac
Revises: 7135f0be454a
Create Date: 2019-10-24 11:13:21.703056

"""
from alembic import op
import sqlalchemy as sa
import sqlalchemy_json


# revision identifiers, used by Alembic.
revision = '5a000ca9abac'
down_revision = '7135f0be454a'
branch_labels = None
depends_on = None


def upgrade():
    op.execute('PRAGMA foreign_keys = OFF')
    op.execute("PRAGMA legacy_alter_table=ON")
    # Make the deleted column from configuration_object not nullable
    op.rename_table('configuration_object', 'configuration_object_old')
    op.create_table('configuration_object',
        sa.Column('created_at', sa.DateTime(), nullable=False),
        sa.Column('updated_at', sa.DateTime(), nullable=False),
        sa.Column('discriminator', sa.TEXT(), nullable=True),
        sa.Column('tag', sa.Enum('ACME_ACCOUNT', 'CERTIFICATE', 'COLLECTION', 'DOMAIN', 'ENDPOINT', 'KEY_CHAIN', 'ROLE', 'STORAGE_GATEWAY', "USER_CREDENTIAL", "WILDCARD_DOMAIN", name='configurationobjecttag'), nullable=False),
        sa.Column('id', sa.TEXT(), nullable=False),
        sa.Column('serial', sa.INTEGER(), nullable=False),
        sa.Column('configuration', sa.TEXT(), nullable=False),
        sa.Column('changed', sa.BOOLEAN(), server_default='0', nullable=False),
        sa.Column('deleted', sa.BOOLEAN(), server_default='0', nullable=False),
        sa.PrimaryKeyConstraint('id')
    )
    op.execute(
	"""
	INSERT INTO configuration_object
	(id, created_at, updated_at, discriminator, tag,
         serial, configuration, changed, deleted)
        SELECT id, created_at, updated_at, discriminator, tag,
         serial, configuration, changed,
         case when deleted is null then 0 else 1 end
         from configuration_object_old
	""")
    op.drop_table('configuration_object_old')
    # Remove the deleted column from storage_gateway, as it is redundant with
    # the configuration_object table
    op.rename_table('storage_gateway', 'storage_gateway_old')
    op.create_table('storage_gateway',
	sa.Column('id', sa.TEXT(), nullable=False),
	sa.Column(
	    'display_name',
	    sa.TEXT(),
	    nullable=False),
	sa.Column(
	    'credential',
	    sqlalchemy_json.NestedMutableJson(),
	    nullable=True),
	sa.Column(
	    'storage_policies',
	    sqlalchemy_json.NestedMutableJson(),
	    nullable=True),
	sa.Column(
	    'allow_guest_collections',
	    sa.BOOLEAN(),
	    server_default='0',
	    nullable=False),
	sa.Column(
	    'allow_mapped_collections',
	    sa.BOOLEAN(),
	    server_default='0',
	    nullable=False),
	sa.Column(
	    'require_high_assurance',
	    sa.BOOLEAN(),
	    server_default='0',
	    nullable=False),
	sa.Column(
	    'connector_id',
	    sa.TEXT(),
	    nullable=False),
	sa.ForeignKeyConstraint(
	    ['connector_id'],
	    ['connector.id'], ),
	sa.ForeignKeyConstraint(
	    ['id'],
	    ['configuration_object.id'],),
	sa.PrimaryKeyConstraint('id'),
	sa.UniqueConstraint('display_name')
    )
    op.execute(
	"""
	INSERT INTO storage_gateway
	(id, display_name, credential, storage_policies,
	 allow_guest_collections, allow_mapped_collections,
	 require_high_assurance, connector_id)
	 SELECT id, display_name, credential, storage_policies,
	 allow_guest_collections, allow_mapped_collections,
	 require_high_assurance, connector_id
	 FROM storage_gateway_old
	""")
    # Remove the backup
    op.drop_table('storage_gateway_old')

    # Remove the deleted column from storage_gateway, as it is redundant with
    # the configuration_object table
    op.rename_table('user_credential', 'user_credential_old')
    op.create_table(
	'user_credential',
	sa.Column('id',
	    sa.TEXT(),
	    nullable=False),
	sa.Column(
	    'display_name',
	    sa.TEXT(),
	    nullable=True),
	sa.Column(
	    'identity_id',
	    sa.TEXT(),
	    nullable=False),
	sa.Column(
	    'storage_gateway_user_id',
	    sa.TEXT(),
	    nullable=False),
	sa.Column(
	    'invalid',
	    sa.Boolean(),
	    server_default='0',
	    nullable=False),
	sa.Column(
	    'storage_gateway_id',
	    sa.TEXT(),
	    nullable=False),
	sa.ForeignKeyConstraint(
	    ['id'],
	    ['configuration_object.id'], ),
	sa.ForeignKeyConstraint(
	    ['storage_gateway_id'],
	    ['storage_gateway.id'], ),
	sa.PrimaryKeyConstraint('id'),
	sa.UniqueConstraint(
	    'storage_gateway_id',
	    'storage_gateway_user_id',
	    'identity_id')
    )
    op.execute(
	"""
	INSERT INTO user_credential (
	    id, display_name, identity_id,
	    storage_gateway_user_id,
	    invalid, storage_gateway_id)
	SELECT id, display_name, identity_id,
	    storage_gateway_user_id,
	    invalid, storage_gateway_id
	FROM user_credential_old
	""")
    op.drop_table("user_credential_old")


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('user_credential', sa.Column('deleted', sa.BOOLEAN(), server_default=sa.text("'0'"), nullable=False))
    op.add_column('storage_gateway', sa.Column('deleted', sa.BOOLEAN(), server_default=sa.text("'0'"), nullable=False))
    # ### end Alembic commands ###
