"""Remove storage_gateway.display_name uniqueness constraint

Revision ID: b8c8667d06e3
Revises: 93e24e66ac6e
Create Date: 2020-03-04 08:02:25.785586

"""
from alembic import op
import sqlalchemy as sa
import sqlalchemy_json


# revision identifiers, used by Alembic.
revision = 'b8c8667d06e3'
down_revision = '93e24e66ac6e'
branch_labels = None
depends_on = None


def upgrade():
    op.execute('PRAGMA foreign_keys = OFF')
    op.execute("PRAGMA legacy_alter_table=ON")

    # Remove the uniqueness constraint on the display_name
    op.rename_table('storage_gateway', 'storage_gateway_old')
    op.create_table(
        'storage_gateway',
        sa.Column('id', sa.TEXT(),
            nullable=False),
        sa.Column(
            'display_name',
            sa.TEXT(),
            nullable=False),
        sa.Column(
            'require_high_assurance',
            sa.BOOLEAN(),
            server_default='0',
            nullable=False),
        sa.Column(
            'connector_id',
            sa.TEXT(),
            nullable=False),
        sa.ForeignKeyConstraint(
            ['id'],
            ['configuration_object.id'],),
        sa.PrimaryKeyConstraint('id')
    )
    op.execute(
	"""
	INSERT INTO storage_gateway
	(id, display_name, require_high_assurance, connector_id)
	 SELECT id, display_name,
	 require_high_assurance, connector_id
	 FROM storage_gateway_old
	""")
    # Remove the backup
    op.drop_table('storage_gateway_old')
    pass


def downgrade():
    op.execute('PRAGMA foreign_keys = OFF')
    op.execute("PRAGMA legacy_alter_table=ON")
    conn = op.get_bind()
    res = conn.execute(
        """
        SELECT display_name, count(1) as c
        FROM storage_gateway
        GROUP BY display_name
        HAVING c > 1
        """)

    for r in res.fetchall():
        raise RuntimeError(
            "Unable to downgrade: non-unique display_name {}".format(r[0]))

    # Restore the uniqueness constraint
    op.rename_table('storage_gateway', 'storage_gateway_old')
    op.create_table(
        'storage_gateway',
        sa.Column('id', sa.TEXT(),
            nullable=False),
        sa.Column(
            'display_name',
            sa.TEXT(),
            nullable=False),
        sa.Column(
            'require_high_assurance',
            sa.BOOLEAN(),
            server_default='0',
            nullable=False),
        sa.Column(
            'connector_id',
            sa.TEXT(),
            nullable=False),
        sa.ForeignKeyConstraint(
            ['id'],
            ['configuration_object.id'],),
        sa.PrimaryKeyConstraint('id'),
        sa.UniqueConstraint('display_name')
    )
    op.execute(
	"""
	INSERT INTO storage_gateway
	(id, display_name,
	 require_high_assurance, connector_id)
	 SELECT id, display_name,
	 require_high_assurance, connector_id
	 FROM storage_gateway_old
	""")
    # Remove the backup
    op.drop_table('storage_gateway_old')
